#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import amqp
from . import api
from . import asynchronous
from . import base
from . import cache
from . import compress
from . import config
from . import controller
from . import crypt
from . import data
from . import defines
from . import exceptions
from . import export
from . import extra
from . import geo
from . import git
from . import http
from . import legacy
from . import log
from . import meta
from . import mock
from . import model
from . import mongo
from . import observer
from . import part
from . import queuing
from . import redisdb
from . import request
from . import scheduler
from . import serialize
from . import session
from . import settings
from . import smtp
from . import storage
from . import structures
from . import typesf
from . import util
from . import validation

from .amqp import AMQP
from .api import Api, OAuthApi, OAuth1Api, OAuth2Api
from .asynchronous import ASYNC_HEADER, AsyncManager, SimpleManager, QueueManager, AwaitWrapper, CoroutineWrapper,\
    AyncgenWrapper, await_wrap, await_yield, ensure_generator, is_coroutine, is_coroutine_object, is_coroutine_native,\
    to_coroutine, wrap_silent, unavailable, is_neo, Future, coroutine, wakeup, sleep, wait, notify, build_future,\
    ensure_async, header_a, ensure_a
from .base import APP, LEVEL, NAME, VERSION, PLATFORM, IDENTIFIER_SHORT, IDENTIFIER_LONG, IDENTIFIER,\
    API_VERSION, BUFFER_SIZE, MAX_LOG_SIZE, MAX_LOG_COUNT, App, APIApp, WebApp, Template, get_app, get_name,\
    get_base_path, get_request, get_session, get_model, get_controller, get_part, get_adapter, get_manager,\
    get_logger, get_level, is_devel, is_safe, to_locale, on_exit
from .cache import Cache, MemoryCache, FileCache, RedisCache
from .compress import Compress
from .config import conf, conf_prefix, conf_suffix, conf_s, conf_d
from .controller import Controller
from .crypt import Cipher, RC4, Spritz
from .data import DataAdapter, MongoAdapter, TinyAdapter, Collection, MongoCollection, TinyCollection
from .defines import ITERABLES, MOBILE_REGEX, TABLET_REGEX, MOBILE_PREFIX_REGEX, BODY_REGEX, TAG_REGEX,\
    EMAIL_REGEX, WINDOWS_LOCALE, SLUG_PERMUTATIONS
from .exceptions import AppierException, OperationalError, SecurityError, AssertionError,\
    ValidationError, NotFoundError, NotImplementedError, BaseInternalError, ValidationInternalError,\
    ValidationMultipleError, HTTPError, APIError, APIAccessError, OAuthAccessError
from .export import ExportManager, MongoEncoder
from .extra import get_a, post_a, put_a, delete_a, patch_a, get_w, post_w, put_w, delete_w, patch_w
from .geo import GeoResolver
from .git import Git
from .http import get_f, get, post, put, delete, HTTPResponse
from .log import MemoryHandler, ThreadFormatter, DummyLogger, rotating_handler, smtp_handler, in_signature
from .meta import Ordered, Indexed
from .mock import MockObject, MockResponse, MockApp
from .model import Model, LocalModel, Field, link, operation, field
from .mongo import Mongo, get_connection, reset_connection, get_db, drop_db, object_id, dumps
from .observer import Observable
from .part import Part
from .queuing import Queue, MemoryQueue, MultiprocessQueue, AMQPQueue
from .redisdb import Redis
from .request import CODE_STRINGS, Request, MockRequest
from .scheduler import Scheduler
from .serialize import serialize_csv, serialize_ics, build_encoder
from .session import Session, MockSession, MemorySession, FileSession, RedisSession, ClientSession
from .settings import DEBUG, USERNAME, PASSWORD
from .smtp import message, message_base, message_netius, smtp_engine, multipart, plain,\
    html, header
from .storage import StorageEngine, BaseEngine, FsEngine
from .structures import OrderedDict
from .typesf import AbstractType, Type, File, Files, ImageFile, ImageFiles, image, images, Reference,\
    reference, References, references, Encrypted, encrypted, secure
from .util import is_iterable, is_mobile, is_tablet, email_parts, email_mime, email_name, email_base,\
    date_to_timestamp, obfuscate, import_pip, ensure_pip, install_pip, install_pip_s, request_json,\
    get_object, resolve_alias, page_types, find_types, norm_object, set_object, leafs, gen_token,\
    html_to_text, camel_to_underscore, camel_to_readable, quote, unquote, call_safe, base_name, base_name_m,\
    parse_cookie, parse_multipart, decode_params, load_form, check_login, ensure_login, dict_merge,\
    cached, private, ensure, delayed, route, error_handler, exception_handler, before_request, after_request,\
    is_detached, sanitize, verify, execute, ctx_locale, ctx_request, FileTuple, BaseThread, JSONEncoder
from .validation import validate, validate_b, validate_e, safe, eq, gt, gte, lt, lte, not_null,\
    not_empty, not_false, is_in, is_lower, is_simple, is_email, is_url, is_regex, field_eq, field_gt,\
    field_gte, field_lt, field_lte, string_gt, string_lt, string_eq, equals, not_past, not_duplicate,\
    all_different, no_self

from .amqp import get_connection as get_amqp
from .amqp import properties as properties_amqp
from .mongo import get_connection as get_mongo
from .mongo import get_db as get_mongo_db
from .mongo import drop_db as drop_mongo_db
from .mongo import object_id as object_id_mongo
from .mongo import dumps as dumps_mongo
from .mongo import serialize as serialize_mongo
from .mongo import directions as directions_mongo
from .redisdb import get_connection as get_redis
from .redisdb import dumps as dumps_redis

HTTPError = exceptions.HTTPError



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import time
import shutil

from . import legacy
from . import common
from . import config
from . import redisdb

class Cache(object):

    def __init__(self, name = "cache"):
        object.__init__(self)
        self.name = name

    def __len__(self):
        return self.length()

    def __getitem__(self, key):
        return self.get_item(key)

    def __setitem__(self, key, value):
        return self.set_item(key, value)

    def __delitem__(self, key):
        return self.delete_item(key)

    def __contains__(self, item):
        return self.contains(item)

    def __nonzero__(self):
        return True

    def __bool__(self):
        return True

    @classmethod
    def new(cls, *args, **kwargs):
        return cls(*args, **kwargs)

    def length(self):
        return 0

    def clear(self):
        pass

    def get(self, key, default = None):
        try: return self.get_item(key)
        except KeyError: return default

    def get_item(self, key):
        raise KeyError("not found")

    def set(self, *args, **kwargs):
        return self.set_item(*args, **kwargs)

    def set_item(self, key, value, expires = None, timeout = None):
        self.mark()

    def delete_item(self, key):
        self.mark()

    def contains(self, key):
        try: self.get_item(key)
        except KeyError: return False
        return True

    def build_value(self, value, expires, timeout):
        if timeout: expires = time.time() + timeout
        return (value, expires)

class MemoryCache(Cache):

    def __init__(self, name = "memory", *args, **kwargs):
        Cache.__init__(self, name = name, *args, **kwargs)
        self.data = {}

    def length(self):
        return self.data.__len__()

    def clear(self):
        self.data.clear()

    def get_item(self, key):
        value, expires = self.data.__getitem__(key)
        if not expires == None and expires < time.time():
            self.delete_item(key)
            return self.get_item(key)
        return value

    def set_item(self, key, value, expires = None, timeout = None):
        value = self.build_value(value, expires, timeout)
        return self.data.__setitem__(key, value)

    def delete_item(self, key):
        return self.data.__delitem__(key)

class FileCache(Cache):

    def __init__(self, name = "file", *args, **kwargs):
        Cache.__init__(self, name = name, *args, **kwargs)
        self.base_path = kwargs.pop("base_path", None)
        self._ensure_path()

    def length(self):
        if not os.path.exists(self.base_path): return 0
        return len(os.listdir(self.base_path))

    def clear(self):
        shutil.rmtree(self.base_path)

    def get_item(self, key):
        file_path = os.path.join(self.base_path, key)
        if not os.path.exists(file_path): raise KeyError("not found")
        if not os.path.exists(file_path + ".expires"): raise KeyError("not found")
        expires = self._read_file(file_path + ".expires")
        expires = int(expires) if expires else None
        if not expires == None and expires < time.time():
            self.delete_item(key)
            return self.get_item(key)
        value = self._read_file(file_path)
        return value

    def set_item(self, key, value, expires = None, timeout = None):
        file_path = os.path.join(self.base_path, key)
        value, expires = self.build_value(value, expires, timeout)
        expires = str(int(expires)) if expires else None
        expires_s = legacy.bytes(expires, force = True) if expires else b""
        self._write_file(file_path, value)
        self._write_file(file_path + ".expires", expires_s)

    def delete_item(self, key):
        file_path = os.path.join(self.base_path, key)
        os.remove(file_path)
        os.remove(file_path + ".expires")

    def _read_file(self, file_path):
        file = open(file_path, "rb")
        try: data = file.read()
        finally: file.close()
        return data

    def _write_file(self, file_path, data):
        self._ensure_exists()
        file = open(file_path, "wb")
        try: file.write(data)
        finally: file.close()

    def _ensure_exists(self):
        if os.path.exists(self.base_path): return
        os.makedirs(self.base_path)

    def _ensure_path(self):
        if self.base_path: return
        app_path = common.base().get_base_path()
        cache_path = os.path.join(app_path, "cache")
        cache_path = config.conf("CACHE_PATH", cache_path)
        self.base_path = cache_path

class RedisCache(Cache):

    def __init__(self, name = "redis", *args, **kwargs):
        Cache.__init__(self, name = name, *args, **kwargs)
        self.redis = redisdb.get_connection()
        self.redis.ping()

    def length(self):
        keys = self.redis.keys()
        return len(keys)

    def clear(self):
        self.redis.flushdb()

    def get_item(self, key):
        if not self.redis.exists(key): raise KeyError("not found")
        return self.redis.get(key)

    def set_item(self, key, value, expires = None, timeout = None):
        if expires: timeout = expires - time.time()
        if timeout and timeout > 0: self.redis.setex(key, value, int(timeout))
        elif timeout: self.redis.delete(key)
        else: self.redis.set(key, value)

    def delete_item(self, key):
        self.redis.delete(key)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import re

ITERABLES = (list, tuple)
""" The tuple that defined the various base types
that are considered to be generally "iterable" """

MOBILE_REGEX = re.compile(r"(android|bb\\d+|meego).+mobile|avantgo|bada\\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino", re.I | re.M)
""" The regular expression that is going to be used
to validate the complete user agent string for mobile """

TABLET_REGEX = re.compile(r"(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino|android|ipad|playbook|silk", re.I | re.M)
""" The regular expression that is going to be used
to validate the complete user agent string for tablet """

MOBILE_PREFIX_REGEX = re.compile(r"1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\\-(n|u)|c55\\/|capi|ccwa|cdm\\-|cell|chtm|cldc|cmd\\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\\-s|devi|dica|dmob|do(c|p)o|ds(12|\\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\\-|_)|g1 u|g560|gene|gf\\-5|g\\-mo|go(\\.w|od)|gr(ad|un)|haie|hcit|hd\\-(m|p|t)|hei\\-|hi(pt|ta)|hp( i|ip)|hs\\-c|ht(c(\\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\\-(20|go|ma)|i230|iac( |\\-|\\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\\/)|klon|kpt |kwc\\-|kyo(c|k)|le(no|xi)|lg( g|\\/(k|l|u)|50|54|\\-[a-w])|libw|lynx|m1\\-w|m3ga|m50\\/|ma(te|ui|xo)|mc(01|21|ca)|m\\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\\-2|po(ck|rt|se)|prox|psio|pt\\-g|qa\\-a|qc(07|12|21|32|60|\\-[2-7]|i\\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\\-|oo|p\\-)|sdk\\/|se(c(\\-|0|1)|47|mc|nd|ri)|sgh\\-|shar|sie(\\-|m)|sk\\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\\-|v\\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\\-|tdg\\-|tel(i|m)|tim\\-|t\\-mo|to(pl|sh)|ts(70|m\\-|m3|m5)|tx\\-9|up(\\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\\-|your|zeto|zte\\-", re.I | re.M)
""" The regular expression to test the base prefix
of the user agent string for mobile browser """

BODY_REGEX = re.compile(r"<body[^<>]*?>(.*?)</body>", re.I)
""" Regular expression that is going to be used in the matching
of the partial contents (child) nodes of a body html node """

TAG_REGEX = re.compile(r"<[^<]*?>")
""" The regular expression that is going to be used in the matching
of an html/xml based node in the html to plain text conversion """

EMAIL_REGEX = re.compile(
    r"((?P<name>^.+) \<(?P<email_a>[\w\d\._%+-]+@[\w\d\.\-]+)\>)|(?P<email_b>[\w\d\._%+-]+@[\w\d\.\-]+)$",
    flags = re.UNICODE
)
""" The regular expression that is going to be used
in the matching of email lines, it supports both the
extended (including name) and the simplified versions """

WINDOWS_LOCALE = dict(
    af = "Afrikaans",
    sq = "Albanian",
    am = "Amharic",
    ar_DZ = "Arabic_Algeria",
    ar_BH = "Arabic_Bahrain",
    ar_EG = "Arabic_Egypt",
    ar_IQ = "Arabic_Iraq",
    ar_JO = "Arabic_Jordan",
    ar_KW = "Arabic_Kuwait",
    ar_LB = "Arabic_Lebanon",
    ar_LY = "Arabic_Libya",
    ar_MA = "Arabic_Morocco",
    ar_OM = "Arabic_Oman",
    ar_QA = "Arabic_Qatar",
    ar_SA = "Arabic_Saudi Arabia",
    ar_SY = "Arabic_Syria",
    ar_TN = "Arabic_Tunisia",
    ar_AE = "Arabic_United Arab Emirates",
    ar_YE = "Arabic_Yemen",
    hy = "Armenian",
    az_AZ = "Azeri_Cyrillic",
    eu = "Basque",
    be = "Belarusian",
    bn = "Bengali_Bangladesh",
    bs = "Bosnian",
    bg = "Bulgarian",
    my = "Burmese",
    ca = "Catalan",
    zh_CN = "Chinese_China",
    zh_HK = "Chinese_Hong Kong SAR",
    zh_MO = "Chinese_Macau SAR",
    zh_SG = "Chinese_Singapore",
    zh_TW = "Chinese_Taiwan",
    hr = "Croatian",
    cs = "Czech",
    da = "Danish",
    dv = "Divehi; Dhivehi; Maldivian",
    nl_BE = "Dutch_Belgium",
    nl_NL = "Dutch_Netherlands",
    en_AU = "English_Australia",
    en_BZ = "English_Belize",
    en_CA = "English_Canada",
    en_CB = "English_Caribbean",
    en_GB = "English_Great Britain",
    en_IN = "English_India",
    en_IE = "English_Ireland",
    en_JM = "English_Jamaica",
    en_NZ = "English_New Zealand",
    en_PH = "English_Phillippines",
    en_ZA = "English_Southern Africa",
    en_TT = "English_Trinidad",
    en_US = "English_United States",
    et = "Estonian",
    fo = "Faroese",
    fa = "Farsi_Persian",
    fi = "Finnish",
    fr_BE = "French_Belgium",
    fr_CA = "French_Canada",
    fr_FR = "French_France",
    fr_LU = "French_Luxembourg",
    fr_CH = "French_Switzerland",
    mk = "FYRO Macedonia",
    gd_IE = "Gaelic_Ireland",
    gd = "Gaelic_Scotland",
    de_AT = "German_Austria",
    de_DE = "German_Germany",
    de_LI = "German_Liechtenstein",
    de_LU = "German_Luxembourg",
    de_CH = "German_Switzerland",
    el = "Greek",
    gn = "Guarani_Paraguay",
    gu = "Gujarati",
    he = "Hebrew",
    hi = "Hindi",
    hu = "Hungarian",
    id = "Indonesian",
    it_IT = "Italian_Italy",
    it_CH = "Italian_Switzerland",
    ja = "Japanese",
    kn = "Kannada",
    ks = "Kashmiri",
    kk = "Kazakh",
    km = "Khmer",
    ko = "Korean",
    lo = "Lao",
    la = "Latin",
    lv = "Latvian",
    lt = "Lithuanian",
    ms_BN = "Malay_Brunei",
    ms_MY = "Malay_Malaysia",
    ml = "Malayalam",
    mt = "Maltese",
    mi = "Maori",
    mr = "Marathi",
    mn = "Mongolian",
    ne = "Nepali",
    no_NO = "Norwegian_Bokml",
    pl = "Polish",
    pt_BR = "Portuguese_Brazil",
    pt_PT = "Portuguese_Portugal",
    pa = "Punjabi",
    rm = "Raeto-Romance",
    ro_MO = "Romanian_Moldova",
    ro = "Romanian_Romania",
    ru = "Russian",
    ru_MO = "Russian_Moldova",
    sa = "Sanskrit",
    sr_SP = "Serbian_Cyrillic",
    tn = "Setsuana",
    sd = "Sindhi",
    si = "Sinhala; Sinhalese",
    sk = "Slovak",
    sl = "Slovenian",
    so = "Somali",
    sb = "Sorbian",
    es_AR = "Spanish_Argentina",
    es_BO = "Spanish_Bolivia",
    es_CL = "Spanish_Chile",
    es_CO = "Spanish_Colombia",
    es_CR = "Spanish_Costa Rica",
    es_DO = "Spanish_Dominican Republic",
    es_EC = "Spanish_Ecuador",
    es_SV = "Spanish_El Salvador",
    es_GT = "Spanish_Guatemala",
    es_HN = "Spanish_Honduras",
    es_MX = "Spanish_Mexico",
    es_NI = "Spanish_Nicaragua",
    es_PA = "Spanish_Panama",
    es_PY = "Spanish_Paraguay",
    es_PE = "Spanish_Peru",
    es_PR = "Spanish_Puerto Rico",
    es_ES = "Spanish_Spain (Traditional)",
    es_UY = "Spanish_Uruguay",
    es_VE = "Spanish_Venezuela",
    sw = "Swahili",
    sv_FI = "Swedish_Finland",
    sv_SE = "Swedish_Sweden",
    tg = "Tajik",
    ta = "Tamil",
    tt = "Tatar",
    te = "Telugu",
    th = "Thai",
    bo = "Tibetan",
    ts = "Tsonga",
    tr = "Turkish",
    tk = "Turkmen",
    uk = "Ukrainian",
    UTF_8 = "Unicode",
    ur = "Urdu",
    uz_UZ = "Uzbek_Cyrillic",
    vi = "Vietnamese",
    cy = "Welsh",
    xh = "Xhosa",
    yi = "Yiddish",
    zu = "Zulu"
)
""" The map containing the associations between the
normalized version/descriptor of the locale and the
longer windows version of them so that it may be used
when setting locales for windows based operative systems """

SLUG_PERMUTATIONS = (
    ("á", "a"),
    ("é", "e"),
    ("í", "i"),
    ("ó", "o"),
    ("ú", "u"),
    ("à", "a"),
    ("ã", "a"),
    ("õ", "o"),
    ("â", "a"),
    ("ê", "e"),
    ("ô", "o"),
    ("ç", "c"),
    ("ä", "a"),
    ("ë", "e"),
    ("ï", "i"),
    ("ö", "o"),
    ("ü", "u"),
    ("ÿ", "y"),
    ("æ", "ae"),
    ("œ", "oe")
)
""" The complete set of permutation tuples to be used in
a slug based simplification process """



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import util

class Observable(object):
    """
    The base class that implements the observable
    patter allowing the object to handle a series of
    event in a dynamic fashion.

    The class should be able to handle both context
    based event (instance level) and global level
    events (class level).

    This class should be friendly to multiple inheritance
    and should avoid variable naming collision.
    """

    _events_g = {}
    """ The dictionary containing the global association
    between the global event names and the handler methods """

    def __init__(self, *args, **kwargs):
        self._events = {}

    @classmethod
    def name_f(cls, name):
        cls_name = cls.__name__
        cls_name = util.camel_to_underscore(cls_name)
        name_f = cls_name + "." + name
        return name_f

    @classmethod
    def bind_g(cls, name, method, oneshot = False):
        if oneshot: method.oneshot = oneshot
        name_f = cls.name_f(name)
        methods = cls._events_g.get(name_f, [])
        methods.append(method)
        cls._events_g[name_f] = methods

    @classmethod
    def unbind_g(cls, name, method = None):
        name_f = cls.name_f(name)
        methods = cls._events_g.get(name_f, [])
        if method: methods.remove(method)
        else: del methods[:]

    @classmethod
    def trigger_g(cls, name, *args, **kwargs):
        oneshots = None
        name_f = cls.name_f(name)
        methods = cls._events_g.get(name_f, [])
        for method in methods:
            method(*args, **kwargs)
            if not hasattr(method, "oneshot"): continue
            if not method.oneshot: continue
            oneshots = [] if oneshots == None else oneshots
            oneshots.append(method)
        if not oneshots: return
        for oneshot in oneshots: cls.unbind_g(name, oneshot)

    def build(self):
        pass

    def destroy(self):
        self.unbind_all()

    def bind(self, name, method, oneshot = False):
        if oneshot: method.oneshot = oneshot
        methods = self._events.get(name, [])
        methods.append(method)
        self._events[name] = methods

    def unbind(self, name, method = None):
        methods = self._events.get(name, [])
        if method: methods.remove(method)
        else: del methods[:]

    def unbind_all(self):
        if not hasattr(self, "_events"): return
        for methods in self._events.values(): del methods[:]
        self._events.clear()

    def trigger(self, name, *args, **kwargs):
        cls = self.__class__
        self.trigger_l(name, cls, *args, **kwargs)

    def trigger_l(self, name, level, *args, **kwargs):
        oneshots = None
        methods = self._events.get(name, [])
        for method in methods:
            method(*args, **kwargs)
            if not hasattr(method, "oneshot"): continue
            if not method.oneshot: continue
            oneshots = [] if oneshots == None else oneshots
            oneshots.append(method)
        level.trigger_g(name, self, *args, **kwargs)
        if not oneshots: return
        for oneshot in oneshots: self.unbind(name, oneshot)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import sys
import threading

from . import config
from . import legacy
from . import exceptions

ASYNC_HEADER = -1

class AsyncManager(object):

    def __init__(self, owner):
        object.__init__(self)
        self.owner = owner

    def start(self):
        pass

    def stop(self):
        pass

    def add(self, method, args, kwargs, request = None, mid = None):
        pass

class SimpleManager(AsyncManager):

    def add(self, method, args, kwargs, request = None, mid = None):
        if request: kwargs["request"] = request
        if mid: kwargs["mid"] = mid
        thread = threading.Thread(
            target = method,
            args = args,
            kwargs = kwargs
        )
        thread.start()

class QueueManager(AsyncManager):

    def __init__(self, owner):
        AsyncManager.__init__(self, owner)
        self.queue = []
        self.condition = threading.Condition()

    def start(self):
        self.thread = threading.Thread(target = self.handler)
        self.running = True
        self.thread.daemon = True
        self.thread.start()

    def stop(self):
        self.running = False

    def add(self, method, args, kwargs, request = None, mid = None):
        if request: kwargs["request"] = request
        if mid: kwargs["mid"] = mid
        item = (method, args, kwargs)
        self.condition.acquire()
        try:
            self.queue.append(item)
            self.condition.notify()
        finally:
            self.condition.release()

    def handler(self):
        while self.running:
            self.condition.acquire()
            while not self.queue: self.condition.wait()
            try: item = self.queue.pop(0)
            finally: self.condition.release()
            method, args, kwargs = item
            try: method(*args, **kwargs)
            except BaseException as exception:
                self.owner.log_error(
                    exception,
                    message = "Problem handling async item: %s"
                )

class AwaitWrapper(object):
    pass

class CoroutineWrapper(object):
    pass

class AyncgenWrapper(object):
    pass

def await_wrap(generator):
    return generator

def await_yield(value):
    yield value

def ensure_generator(value):
    if legacy.is_generator(value): return True, value
    return False, value

def is_coroutine(callable):
    if hasattr(callable, "_is_coroutine"): return True
    return False

def is_coroutine_object(generator):
    if legacy.is_generator(generator): return True
    return False

def is_coroutine_native(generator):
    return False

def to_coroutine(callable, *args, **kwargs):
    """
    Converts the provided (callback based) callable into a coroutine
    like object/function that is able to yield a future to be notified
    by a wrapper around the "original" callback.

    The parameters to be used in the calling should be passed as list
    and keyword based arguments.

    :type callable: Callable
    :param callable: The callable that is going to be converted into
    a coroutine.
    :rtype: Future
    :return: The future object that is going to be used in the control
    of the coroutine execution.
    """

    # tries to retrieve both the future and a callback from
    # the provided key based arguments in case thre's no future
    # a new one is created for the current context as for the
    # callback an unset one is used for invalid situations
    future = kwargs.pop("future", None) or build_future()
    callback = kwargs.get("callback", None)

    def callback_wrap(result, *args, **kwargs):
        # sets the final result in the associated future
        # this should contain the contents coming from
        # the callback operation (payload)
        future.set_result(result)

        # in case the "original" callback is set calls it
        # with the provided arguments as expected
        callback and callback(result, *args, **kwargs)

        # runs the wake up operation so that the main loop
        # associated with the current execution environment
        # is able to process the new future result
        wakeup()

    # sets the wrapped callback in the key based arguments and
    # then runs the callback with the new arguments, yielding
    # the future that has just been created, effectively creating
    # a coroutine interface from a callback one
    kwargs["callback"] = callback_wrap
    callable(*args, **kwargs)
    yield future

def wrap_silent(function):
    return function

def unavailable(*args, **kwargs):
    raise exceptions.AppierException(
        message = "No support for async available"
    )

def is_neo():
    return sys.version_info[0] >= 3 and sys.version_info[1] >= 3

def header_a_():
    yield ASYNC_HEADER

def ensure_a_(*args, **kwargs):
    yield ensure_async(*args, **kwargs)

# determines the target service configuration that is
# going to be used, this is going to be used to create
# the proper adaptation for the async models
server = config.conf("SERVER", None)
if server == "netius":
    import netius
    Future = netius.Future
    coroutine = netius.coroutine
    wakeup = netius.wakeup
    sleep = netius.sleep
    wait = netius.wait
    notify = netius.notify
    build_future = netius.build_future
    ensure_async = netius.ensure
else:
    Future = unavailable
    coroutine = wrap_silent
    wakeup = unavailable
    sleep = unavailable
    wait = unavailable
    notify = unavailable
    build_future = unavailable
    ensure_async = unavailable



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import config
from . import legacy

try: import pika
except: pika = None

URL = "amqp://guest:guest@localhost"
""" The default url to be used for the connection when
no other url is provided (used most of the times) """

TIMEOUT = 100
""" The time the retrieval of a connection waits before
returning this avoid possible problems with the current
implementation of the blocking client """

connection = None
""" The global wide connection to the amqp server
that is meant to be used across sessions """

class AMQP(object):

    def __init__(self, url = None):
        self.url = url
        self._connection = None

    def get_connection(self, url = None, timeout = TIMEOUT):
        if self._connection: return self._connection
        url_c = config.conf("AMQP_URL", None)
        url_c = config.conf("CLOUDAMQP_URL", url_c)
        url_c = config.conf("RABBITMQ_URL", url_c)
        url = url or self.url or url_c or URL
        url_p = legacy.urlparse(url)
        parameters = pika.ConnectionParameters(
            host = url_p.hostname,
            virtual_host = url_p.path[1:],
            credentials = pika.PlainCredentials(url_p.username, url_p.password)
        )
        parameters.socket_timeout = timeout
        self._connection = pika.BlockingConnection(parameters)
        self._connection = _set_fixes(self._connection)
        return self._connection

def get_connection(url = URL, timeout = TIMEOUT):
    global connection
    url = config.conf("AMQP_URL", url)
    url = config.conf("CLOUDAMQP_URL", url)
    url = config.conf("RABBITMQ_URL", url)
    url_p = legacy.urlparse(url)
    parameters = pika.ConnectionParameters(
        host = url_p.hostname,
        virtual_host = url_p.path[1:],
        credentials = pika.PlainCredentials(url_p.username, url_p.password)
    )
    parameters.socket_timeout = timeout
    connection = pika.BlockingConnection(parameters)
    connection = _set_fixes(connection)
    return connection

def properties(*args, **kwargs):
    return pika.BasicProperties(*args, **kwargs)

def _set_fixes(connection):
    def disconnect():
        connection.socket.close()

    if not hasattr(connection, "disconnect"):
        connection.disconnect = disconnect
    return connection
